##
# File:    ConfigInfoDataSet.py
# Date:    17-Mar-2016
#
# Updates:
#  06-Apr-2016  jdw  get default id ranges from ConfigInfoData -
#  07-Apr-2016  jdw  add resource data file to hold location exceptions.
#  15-Aug-2016  jdw  add support backup sites serving as surrogates for production sites.
#  19-Aug-2016  jdw  add getDefaultSiteId()
#  23-Aug-2016  jdw  add getDataSetLocations() and writeLocationList() and getDataSetLocationDict() and removeDataSets()
#  01-Jan-2018  ep   add getTestIdRange()
##
"""
Provides accessors for the correspondence between deposition data identifiers and
deposition and annotation sites (e.g. wwpdb_site_id).

"""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import datetime
import json
import logging
import sys

from oslo_concurrency import lockutils

from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppDepUI

logger = logging.getLogger(__name__)


class ConfigInfoDataSet:
    """
    Provides accessors for the correspondence between deposition data identifiers and
    deposition and annotation sites (e.g. wwpdb_site_id).

    """

    def __init__(self, verbose=False, log=sys.stderr):  # noqa: ARG002 pylint: disable=unused-argument
        self.__verbose = verbose
        self.__debug = True
        self.__cI = ConfigInfo(siteId=None, verbose=self.__verbose)
        self.__cIDepUI = ConfigInfoAppDepUI(siteId=getSiteId())
        # Default data set id range assignments
        self.__depIdAssignments = self.__cI.get("SITE_DATASET_ID_ASSIGNMENT_DICTIONARY")
        self.__depTestIdAssignments = self.__cI.get("SITE_DATASET_TEST_ID_ASSIGNMENT_DICTIONARY")
        self.__siteBackupD = self.__cI.get("SITE_BACKUP_DICT", default={})
        self.__dsLocD = None
        self.__lockDirPath = self.__cI.get("SITE_SERVICE_REGISTRATION_LOCKDIR_PATH", "/tmp")  # noqa: S108
        lockutils.set_defaults(self.__lockDirPath)

    def getSiteId(self, depSetId):
        """Return siteId for the input depSetId subject to site backup details -

        siteBackupD[prodSite] = [backupSite1, backupSite2,...]
        """
        siteId = self.__getSiteId(depSetId)
        mySiteId = self.__cI.get("SITE_PREFIX", default=None)
        if mySiteId and siteId:
            # is mySiteId a backup for siteId?
            if siteId in self.__siteBackupD and mySiteId in self.__siteBackupD[siteId]:
                if self.__debug:
                    logger.debug("using backup %s for %s", mySiteId, siteId)
                siteId = mySiteId

        return siteId

    def getDataSetLocationDict(self):
        d = {}
        try:
            d = self.__readLocationDictionary()
            return d
        except Exception as e:
            logger.error("failed reading data set location dictionary: %s", str(e))
            if self.__debug:
                logger.exception("failed reading data set location dictionary")
        return d

    def getDataSetLocations(self, siteId):
        dsL = []
        try:
            d = self.__readLocationDictionary()
            for ky in d:
                if d[ky] == siteId:
                    dsL.append(ky)
            return dsL
        except Exception as e:
            logger.info("failed reading data set locations for site %r - %s", siteId, str(e))
            if self.__debug:
                logger.exception("failed reading data set locations for site %r - %s", siteId, str(e))
        return []

    def removeDataSets(self, dataSetIdList):
        try:
            d = self.__readLocationDictionary()
            for dsId in dataSetIdList:
                if dsId in d:
                    del d[dsId]
            return self.__writeLocationDictionary(d)
        except Exception as e:
            logger.error("failed %s", str(e))
            if self.__debug:
                logger.exception("failed")
        return False

    def writeLocationList(self, siteId, dataSetIdList):
        try:
            d = self.__readLocationDictionary()
            for dsId in dataSetIdList:
                d[dsId] = siteId
            return self.__writeLocationDictionary(d)
        except Exception as e:
            logger.error("failed data set locations for site %r - %s", siteId, str(e))
            if self.__debug:
                logger.exception("failed data set locations for site %rs", siteId)
        return False

    def __readLocationDictionary(self):
        """Read the dictionary cotaining data set site location information.

        Returns: d[<data_set_id>] = <site_id> or a empty dictionary.
        """
        fp = self.__cIDepUI.get_site_dataset_siteloc_file_path()
        try:
            with open(fp) as infile:
                return json.load(infile)
        except Exception as e:
            logger.error("failed reading json resource file %s - %s", fp, str(e))
            if self.__debug:
                logger.exception("failed reading json resource file %s", fp)
        return {}

    @lockutils.synchronized("configdataset.exceptionfile-lock", external=True)
    def __writeLocationDictionary(self, dsLocD, backup=True):
        """Write the input dictionary cotaining exceptional data set to site correspondences,

        Returns: True for success or False otherwise
        """
        fp = self.__cIDepUI.get_site_dataset_siteloc_file_path()

        try:
            if backup:
                bp = fp + datetime.datetime.now().strftime("-%Y-%m-%d-%H-%M-%S")  # noqa: DTZ005
                d = self.__readLocationDictionary()
                with open(bp, "w") as outfile:
                    json.dump(d, outfile, indent=4)
            with open(fp, "w") as outfile:
                json.dump(dsLocD, outfile, indent=4)
            return True
        except Exception as e:
            logger.error("failed writing json resource file %s - %s", fp, str(e))
            if self.__debug:
                logger.exception("failed writing json resource file %s", fp)
        return False

    def getDefaultIdRange(self, siteId):
        """Return the default upper and lower deposition data set identifier codes
        assigned to the input siteId.

        Any site lacking a default range will get the range assigned to the UNASSIGNED site.

        Returns:   (lower bound, upper bound) for data set identifiers (int)
        """
        if siteId in self.__depIdAssignments:
            DEPID_START, DEPID_STOP = self.__depIdAssignments[siteId]
        elif "UNASSIGNED" in self.__depIdAssignments:
            DEPID_START, DEPID_STOP = self.__depIdAssignments["UNASSIGNED"]
        else:
            DEPID_START, DEPID_STOP = (-1, -1)
        return (DEPID_START, DEPID_STOP)

    def getTestIdRange(self, siteId):
        """Return the upper and lower deposition data set identifier codes
        assigned to the input siteId.

        Any site lacking a default range will get the range (-1, -1)

        Returns:   (lower bound, upper bound) for data set identifiers (int)
        """
        if siteId in self.__depTestIdAssignments:
            DEPID_START, DEPID_STOP = self.__depTestIdAssignments[siteId]
        else:
            DEPID_START, DEPID_STOP = (-1, -1)
        return (DEPID_START, DEPID_STOP)

    def getDefaultSiteId(self, depSetId):
        """Get the default site assignment for the input data set id."""
        return self.__getSiteId(depSetId)

    def __getSiteId(self, depSetId):
        """Return the siteId to which the input depSetId is within the default
        code assignment range.

        Input may be either a string "D_xxxxxxxxxx" or an integer/string "xxxxxxxxxx".

        """
        # check for exceptional cases --
        try:
            if self.__dsLocD is None:
                self.__dsLocD = self.__readLocationDictionary()
            if str(depSetId)[:2] == "D_":
                if depSetId in self.__dsLocD:
                    return self.__dsLocD[depSetId]
            else:
                tId = "D_" + str("%010d" % int(depSetId))
                if tId in self.__dsLocD:
                    return self.__dsLocD[tId]
        except ValueError as e:
            # From trying to take int
            if self.__debug:
                logger.error("checking deposition conversion for %r - %s", depSetId, str(e))
                return None
        except Exception as e:
            if self.__debug:
                logger.exception("failed checking for exception dictionary for %r - %s", depSetId, str(e))
        #
        # check default range assignment --
        try:
            if str(depSetId).startswith("D_"):
                idVal = int(str(depSetId)[2:])
            else:
                idVal = int(str(depSetId))
            for ky in self.__depIdAssignments:
                idMin, idMax = self.__depIdAssignments[ky]
                if (idVal >= idMin) and (idVal <= idMax):
                    return ky
        except ValueError as e:
            # From trying to take int
            if self.__debug:
                logger.error("checking deposition conversion for %r - %s", depSetId, str(e))
        except Exception as e:
            if self.__debug:
                logger.exception("failed checking deposition range for %r - %s", depSetId, str(e))
        return None



#!/usr/bin/env python
##
# File:    ConfigInfoFileExec.py
# Author:  jdw
# Date:    5-Apr-2016
# Version: 0.001
#
# Updates:
#   6-Apr-2016  jdw   add fallback resources describing locations and sites -
#   7-Apr-2016  jdw   add directory existence checks
#  10-May-2016  jdw   add support for additional private configuration sections.
#  14-May-2016  jdw   add install section and cache update by location
#   4-Dec-2016  jdw   new CLI option for adding extra option section names added to common name space
#                     The defaults for common sections are 'database_services' and 'validation_module'.
#
#                     The defaults for private namespaces include 'os_environment', 'httpd_services',
#                     and 'install_environment'
#  12-Apr-2017  jdw   add missing site-common and common configuration paths in search for private configuration sections.
#  11-Oct-2017  jdw   add back_server_* private section
#  05-Oct-2018   ep   add options to inject a mock environment variable into config. Add support for a R/O source tree and R/W cache directory
"""
Execuction wrapper for configuration option and cache file management.

"""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.001"

import logging
import os
import sys
import traceback
from optparse import SUPPRESS_HELP, OptionParser  # pylint: disable=deprecated-module

from wwpdb.utils.config.ConfigInfoFile import ConfigInfoFile

logging.basicConfig(level=logging.INFO, format="[%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()
logger.setLevel(logging.INFO)


class ConfigInfoFileExec:
    """
    Execuction wrapper for configuration option and cache file management.

    """

    def __init__(self, mockTopPath=None, sourceDirPath=None, verbose=True, log=sys.stderr):
        self.__lfh = log
        self.__verbose = verbose
        self.__debug = False
        self.__mockTopPath = mockTopPath
        self.__topConfigPath = os.getenv("TOP_WWPDB_SITE_CONFIG_DIR", default=None)
        if sourceDirPath:
            self.__sourceDirPath = sourceDirPath
        else:
            self.__sourceDirPath = self.__topConfigPath

        # Complete list of sections maintained as private namespaces
        self.__privateSectionNameList = []
        # additional configuration sections added to the common namespace
        self.__extraCommonSectionNameList = []

    def setPrivateSectionNames(self, sectionNameList):
        self.__privateSectionNameList = sectionNameList

    def __getPrivateSectionNames(self):
        return self.__privateSectionNameList

    def addCommonSectionNames(self, sectionNameList):
        self.__extraCommonSectionNameList = sectionNameList

    def __getExtraCommonSectionNames(self):
        return self.__extraCommonSectionNameList

    def testConfigPath(self, accessType="read"):
        ok = True
        try:
            if self.__topConfigPath is None:
                ok = False
                self.__lfh.write("WARNING - TOP_WWPDB_SITE_CONFIG_DIR is not set in the environment.\n")
            elif accessType == "write" and not os.access(self.__topConfigPath, os.W_OK):
                ok = False
                self.__lfh.write("WARNING - %s lacks write access.\n" % self.__topConfigPath)
            elif accessType == "read" and not os.access(self.__topConfigPath, os.R_OK):
                ok = False
                self.__lfh.write("WARNING - %s lacks read access.\n" % self.__topConfigPath)
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("testConfigPath failing %r\n" % str(e))
            traceback.print_exc(file=self.__lfh)
            ok = False

        return ok

    def __getCommonConfigPath(self, sectionName="common", context="common"):
        cfPath = os.path.join(self.__sourceDirPath, "common", "common.cfg")
        return cfPath, sectionName, context

    def __getSiteCommonConfigPath(self, siteLoc, sectionName="site_common", context="common"):
        cfPath = os.path.join(self.__sourceDirPath, siteLoc.lower(), "site_common", "common.cfg")
        return cfPath, sectionName, context

    def __getSiteConfigPath(self, siteLoc, siteId, sectionName, context="common"):
        cfPath = os.path.join(self.__sourceDirPath, siteLoc.lower(), siteId.lower(), "site.cfg")
        return cfPath, sectionName, context

    def __getSitePythonCachePath(self, siteLoc, siteId):
        cfPath = os.path.join(self.__topConfigPath, siteLoc.lower(), siteId.lower(), "ConfigInfoFileCache.py")
        return cfPath

    def __getSiteJsonCachePath(self, siteLoc, siteId):
        cfPath = os.path.join(self.__topConfigPath, siteLoc.lower(), siteId.lower(), "ConfigInfoFileCache.json")
        return cfPath

    def __getCommonConfig(self):
        """Return the project common configuration options as a dictionary.

        Really depends on config_as_object setting to deserialize dictionary ...
        """
        cD = {}
        try:
            cfPath, sectionName, _context = self.__getCommonConfigPath(sectionName="common", context="common")  # pylint: disable=unused-variable
            cf = ConfigInfoFile(mockTopPath=self.__mockTopPath, verbose=self.__verbose, log=self.__lfh)
            tD = cf.readConfig(configFilePath=cfPath)
            if sectionName.upper() in tD:
                cD = cf.deserializeConfig(tD[sectionName.upper()], optionD=tD[sectionName.upper()])
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("__getCommonConfig failing %r\n" % str(e))
            traceback.print_exc(file=self.__lfh)

        return cD

    def __getConfigPathSectionList(self, siteLoc, siteId, extraCommonSectionNameList, privateSectionNameList):
        """Returns the search path of sections and configuration file paths for the input location and site.
        The site specific configuration file path is always included.   The site-common or project common
        configuration files are included only if these exist.

        Any section names in extraCommonSectionNameList are added to path list for each configuration file -

        Returns: [(configPath,sectionName,context), (configPath,sectionName),context), ...]
        """
        cfPathSectionList = []
        if self.__topConfigPath is not None and siteId is not None and siteLoc is not None:
            (p, s, c) = self.__getSiteConfigPath(siteLoc=siteLoc, siteId=siteId, sectionName=siteId.lower())
            if p is not None and os.access(p, os.R_OK):
                cfPathSectionList.append((p, s, c))
                for cSec in extraCommonSectionNameList:
                    cfPathSectionList.append((p, cSec, "common"))
            (p, s, c) = self.__getSiteCommonConfigPath(siteLoc=siteLoc, sectionName="site_common")
            if p is not None and os.access(p, os.R_OK):
                cfPathSectionList.append((p, s, c))
                for cSec in extraCommonSectionNameList:
                    cfPathSectionList.append((p, cSec, "common"))
            (p, s, c) = self.__getCommonConfigPath(sectionName="common")
            if p is not None and os.access(p, os.R_OK):
                cfPathSectionList.append((p, s, c))
                for cSec in extraCommonSectionNameList:
                    cfPathSectionList.append((p, cSec, "common"))
            #
            # Additional context specific (private) configuration sections - stored in the site, site-common and common config paths
            #
            for sectionName in privateSectionNameList:
                (p, s, c) = self.__getSiteConfigPath(
                    siteLoc=siteLoc, siteId=siteId, sectionName=sectionName, context="private"
                )
                if p is not None and os.access(p, os.R_OK):
                    cfPathSectionList.append((p, s, c))
                (p, s, c) = self.__getSiteCommonConfigPath(siteLoc=siteLoc, sectionName=sectionName, context="private")
                if p is not None and os.access(p, os.R_OK):
                    cfPathSectionList.append((p, s, c))

                (p, s, c) = self.__getCommonConfigPath(sectionName=sectionName, context="private")
                if p is not None and os.access(p, os.R_OK):
                    cfPathSectionList.append((p, s, c))

        return cfPathSectionList

    def __getSiteConfig(self, siteLoc, siteId, deserialize=True):
        """Return the complete site of configuration options for the input location and site."""
        cD = {}
        try:
            privateSectionNameList = self.__getPrivateSectionNames()
            extraCommonSectionNameList = self.__getExtraCommonSectionNames()
            pathSectList = self.__getConfigPathSectionList(
                siteLoc, siteId, extraCommonSectionNameList, privateSectionNameList
            )
            if self.__debug:
                self.__lfh.write("__getSiteConfig Path list for location %r site %r\n" % (siteLoc, siteId))
                for pTup in pathSectList:
                    self.__lfh.write("__getSiteConfig %r\n" % pTup)
            cf = ConfigInfoFile(mockTopPath=self.__mockTopPath, verbose=self.__verbose, log=self.__lfh)
            cD = cf.readConfigFileList(configPathSectionList=pathSectList)
            if deserialize:
                cD = cf.deserializeConfig(cD, optionD=cD)
                # Deserialize any subsections -  avoid checking for private section name with wildcards.
                if True:  # pylint: disable=using-constant-test
                    for k, v in cD.items():
                        if isinstance(v, dict):
                            cD[k] = cf.deserializeConfig(cD[k], optionD=cD)
                else:
                    for sectionName in privateSectionNameList:
                        sU = sectionName.upper()
                        if sU in cD:
                            cD[sU] = cf.deserializeConfig(cD[sU], optionD=cD[sU])
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("__getSiteConfig failing for location %r site %r - %r\n" % (siteLoc, siteId, str(e)))
            traceback.print_exc(file=self.__lfh)
        return cD

    def checkConfig(self, siteLoc, siteId, deserialize=True):
        """Perform sanity checks for the configuration options for the input location and site."""
        try:
            cD = self.__getSiteConfig(siteLoc, siteId, deserialize=deserialize)
            self.__lfh.write("read %d options for location %r site %r\n" % (len(cD), siteLoc, siteId))
            #
            #  - path check -
            deployPath = cD["SITE_DEPLOY_PATH"]
            for k in sorted(cD.keys()):
                v = cD[k]
                if v is None:
                    self.__lfh.write("location %s siteId %s option %s is None\n" % (siteLoc, siteId, k))
                elif not isinstance(v, str):
                    self.__lfh.write("location %s siteId %s option %s is %s\n" % (siteLoc, siteId, k, type(v)))
                elif len(v) < 1:
                    self.__lfh.write("location %s siteId %s option %s is blank\n" % (siteLoc, siteId, k))
                elif v.startswith(deployPath) and not os.access(v, os.R_OK):
                    self.__lfh.write("location %s siteId %s path access error %s\n" % (siteLoc, siteId, v))

        except Exception as e:  # noqa: BLE001
            self.__lfh.write("checkConfig for location %r site %r - %r\n" % (siteLoc, siteId, str(e)))
            traceback.print_exc(file=self.__lfh)

    def printConfig(self, siteLoc, siteId, deserialize=True):
        """Print the configuration options for the input location and site."""
        try:
            cD = self.__getSiteConfig(siteLoc, siteId, deserialize=deserialize)
            self.__lfh.write("read %d options for location %r site %r\n" % (len(cD), siteLoc, siteId))
            for k in sorted(cD.keys()):
                v = cD[k]
                if type(v) == dict:  # noqa: E721
                    self.__lfh.write(" +++ %-45s  %r\n" % (k, "private context"))
                    for k1 in sorted(v.keys()):
                        self.__lfh.write(" ---  --- +++ %-45s  %r\n" % (k1, v[k1]))
                else:
                    self.__lfh.write(" +++ %-45s  %r\n" % (k, v))
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("printConfig failing for location %r site %r - %r\n" % (siteLoc, siteId, str(e)))
            traceback.print_exc(file=self.__lfh)

    def writeConfigCache(self, siteLoc, siteId, skipEmpty=True):
        """Write Python and JSON format cache files using the configuration options for input location and site."""
        self.__lfh.write("Starting writeConfigCache\n")
        try:
            cD = self.__getSiteConfig(siteLoc, siteId, deserialize=True)
            if ((cD is None) or (len(cD) < 1)) and skipEmpty:
                self.__lfh.write("SKIPPING update of empty cache files for location %r site %r\n" % (siteLoc, siteId))
                return False
            cf = ConfigInfoFile(mockTopPath=self.__mockTopPath, verbose=self.__verbose, log=self.__lfh)
            cfCachePath = self.__getSitePythonCachePath(siteLoc, siteId)
            cf.writePythonConfigCache(cacheD={siteId.upper(): cD}, cacheFilePath=cfCachePath)
            cfCachePath = self.__getSiteJsonCachePath(siteLoc, siteId)
            cf.writeJsonConfigCache(cacheD={siteId.upper(): cD}, cacheFilePath=cfCachePath)
            self.__lfh.write(
                "updating cache files with %d options for location %r site %r\n" % (len(cD), siteLoc, siteId)
            )
            return True
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("writeConfigCache failing for location %r site %r - %r\n" % (siteLoc, siteId, str(e)))
            traceback.print_exc(file=self.__lfh)

        return False

    def writeLocationConfigCache(self, siteLoc, skipEmpty=True):
        """Write Python and JSON format cache files using the configuration options for input location and site."""
        self.__lfh.write("Starting writeLocationConfigCache\n")
        try:
            siteD = self.__getLocSiteD()
            siteIdList = []
            if siteLoc.upper() in siteD:
                siteIdList = siteD[siteLoc.upper()]
            for siteId in siteIdList:
                cD = self.__getSiteConfig(siteLoc, siteId, deserialize=True)
                if ((cD is None) or (len(cD) < 1)) and skipEmpty:
                    self.__lfh.write(
                        "SKIPPING update of empty cache files for location %r site %r\n" % (siteLoc, siteId)
                    )
                    continue
                cf = ConfigInfoFile(mockTopPath=self.__mockTopPath, verbose=self.__verbose, log=self.__lfh)
                cfPath = self.__getSiteConfigPath(siteLoc, siteId, "none")[0]
                if os.path.exists(cfPath):
                    cfCachePath = self.__getSitePythonCachePath(siteLoc, siteId)
                    cf.writePythonConfigCache(cacheD={siteId.upper(): cD}, cacheFilePath=cfCachePath)
                    cfCachePath = self.__getSiteJsonCachePath(siteLoc, siteId)
                    cf.writeJsonConfigCache(cacheD={siteId.upper(): cD}, cacheFilePath=cfCachePath)
                    self.__lfh.write(
                        "updating cache files with %d options for location %r site %r\n" % (len(cD), siteLoc, siteId)
                    )
                else:
                    self.__lfh.write(
                        "skipping cache files with for location %r site %r as site.cfg missing %s \n"
                        % (siteLoc, siteId, cfPath)
                    )
            return True
        except Exception as e:  # noqa: BLE001
            self.__lfh.write(
                "writeLocationConfigCache failing for location %r site %r - %r\n" % (siteLoc, siteId, str(e))
            )
            traceback.print_exc(file=self.__lfh)

        return False

    def __getLocSiteD(self):
        # Fetch custom location site details from the global common configuration file -
        comD = self.__getCommonConfig()
        siteD = {}
        if "SITE_LOCATION_SITE_DICT" in comD:
            siteD = comD["SITE_LOCATION_SITE_DICT"]
        else:
            # fallback resources --
            siteD = {
                "RCSB-WEST": ["WWPDB_DEPLOY_PRODUCTION_UCSD"],
                "PDBJ": ["WWPDB_DEPLOY_INTERNAL_PDBJ", "WWPDB_DEPLOY_PRODUCTION_PDBJ"],
                "RCSB-EAST": [
                    "WWPDB_DEPLOY_PRODUCTION_RU",
                    "WWPDB_DEPLOY_VALSRV_RU",
                    "WWPDB_DEPLOY_TEST_RU",
                    "WWPDB_DEPLOY_STAGING_RU",
                    "WWPDB_DEPLOY_ALPHA_RU",
                    "WWPDB_DEPLOY_BETA_RU",
                    "WWPDB_DEPLOY_NEXT_RU",
                    "WWPDB_DEPLOY_INTERNAL_RU",
                    "WWPDB_DEPLOY_DEVEL_RU",
                    "WWPDB_DEPLOY_MACOSX",
                    "WWPDB_DEPLOY_DEVEL_RU",
                    "WWPDB_DEPLOY_DEVEL2_RU",
                    "WWPDB_DEPLOY_DEVEL3_RU",
                    "WWPDB_DEPLOY_DEVEL4_RU",
                    "WWPDB_DEPLOY_DEPGRP1_RU",
                    "WWPDB_DEPLOY_DEPGRP2_RU",
                ],
                "PDBE": ["PDBE_PROD", "PDBE_DEV", "PDBE_LOCAL", "PDBE_HAPPY"],
            }
        return siteD


def main():  # pragma: no cover
    usage = """usage: %prog [options]

    Examples:

     Check a site configuration options file (requires both --locid & --siteid):

       python %prog --check --siteid=WWPDB_DEPLOY_TEST_RU --locid=rcsb-east

     Print the options in the specified site configuration file (requires both --locid & --siteid):

       python %prog --print --siteid=WWPDB_DEPLOY_TEST_RU --locid=rcsb-east

     Write/update the Python and JSON cache files for the specified site (requires --locid or both --locid & --siteid).

       python %prog --writecache --siteid=WWPDB_DEPLOY_TEST_RU --locid=rcsb-east

     Include additional locally scoped configuration sections using --sections="sec1,sec2,..." that
     will be stored in embedded dictionaries using section name keys (default=os_environment,httpd_services)

    """
    parser = OptionParser(usage)

    parser.add_option(
        "--check",
        dest="checkConfig",
        action="store_true",
        default=False,
        help="Check configuration file for a site (--siteid) within a location (--locid)",
    )
    parser.add_option(
        "--print",
        dest="printConfig",
        action="store_true",
        default=False,
        help="Print the configuration options a site (--siteid) within a location (--locid)",
    )
    parser.add_option(
        "--writecache",
        dest="writeCache",
        action="store_true",
        default=False,
        help="Write configuration cache file for a site (--siteid) within a location (--locid)",
    )

    parser.add_option("--siteid", dest="siteId", default=None, help="wwPDB site ID (e.g. WWPDB_DEPLOY_TEST_RU)")
    parser.add_option(
        "--locid", dest="locId", default=None, help="wwPDB location ID (e.g. pdbe, pdbj, rcsb-east, ... )"
    )

    parser.add_option(
        "--sections",
        dest="privateSectionNames",
        default=None,
        help="Comma separated list of private section names (stored as dictionaries under their section name)",
    )
    parser.add_option(
        "--add_common_sections",
        dest="commonSectionNames",
        default=None,
        help="Comma separated list of additional option section names added to the common namespace",
    )
    parser.add_option("-v", "--verbose", default=True, action="store_true", dest="verbose")

    # Test setup configuration
    # mockdir path
    parser.add_option("--mockdir", default=None, help=SUPPRESS_HELP)
    # source dir - specifies a read only source tree to use for finding configuration files
    parser.add_option("--sourcedir", default=None, help=SUPPRESS_HELP)

    (options, _args) = parser.parse_args()  # pylint: disable=unused-variable

    cI = ConfigInfoFileExec(
        mockTopPath=options.mockdir, sourceDirPath=options.sourcedir, verbose=options.verbose, log=sys.stderr
    )
    if options.privateSectionNames is not None:
        privateSectionNameList = [str(x).strip() for x in options.privateSectionNames.split(",")]
    else:
        privateSectionNameList = [
            "os_environment",
            "httpd_services",
            "install_environment",
            "database_services",
            "validation_services",
            "host_site_defaults",
            "test_setup_*",
            "backup_server_*",
        ]
    cI.setPrivateSectionNames(sectionNameList=privateSectionNameList)

    if options.commonSectionNames is not None:
        commonSectionNameList = [str(x).strip() for x in options.commonSectionNames.split(",")]
    else:
        commonSectionNameList = ["database_services", "validation_services"]
    cI.addCommonSectionNames(sectionNameList=commonSectionNameList)

    if (
        options.checkConfig
        and options.siteId is not None
        and options.locId is not None
        and cI.testConfigPath(accessType="read")
    ):
        cI.checkConfig(siteLoc=options.locId, siteId=options.siteId)

    if (
        options.printConfig
        and options.siteId is not None
        and options.locId is not None
        and cI.testConfigPath(accessType="read")
    ):
        cI.printConfig(siteLoc=options.locId, siteId=options.siteId)

    if (
        options.writeCache
        and options.siteId is not None
        and options.locId is not None
        and cI.testConfigPath(accessType="write")
    ):
        cI.writeConfigCache(siteLoc=options.locId, siteId=options.siteId)
    elif (
        options.writeCache
        and options.siteId is None
        and options.locId is not None
        and cI.testConfigPath(accessType="write")
    ):
        cI.writeLocationConfigCache(siteLoc=options.locId)


if __name__ == "__main__":
    main()



#!/usr/bin/env python  # noqa: EXE001
##
# File:    ConfigInfoShellExec.py
# Author:  jdw
# Date:    8-May-2016
# Version: 0.001
#
# Updates:
#      14-May-2016 jdw add support for installation environment
#      16-May-2016 jdw add bootstrap support for reading flat config files directly --
#       4-Dec-2016 jdw add support for extra common configuration sections
#       5-Dec-2016 jdw add export for validation and database services -
##
"""
Execuction wrapper for shell configuration using project configuration files.

"""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.001"

import os
import sys
import traceback

if sys.version_info[0] > 2:
    import importlib
else:
    import imp  # pylint: disable=deprecated-module
import ast

try:
    import ConfigParser  # type: ignore[import-not-found]
except ImportError:
    import configparser as ConfigParser  # noqa: N812
from optparse import OptionParser  # pylint: disable=deprecated-module


class ConfigInfoShellExec:
    """
    Execuction wrapper for shell configuration using project configuration files.

    Shell configuration:

    + by host as argument (fqdn)

        expects  <TOP_WWPDB_SITE_CONFIG_DIR>/common/common.cfg  containing mapping data

    + by site using explicit arguments for siteLoc and siteId -

    both require: TOP_WWPDB_SITE_CONFIG_DIR in the environment or as argument


    """

    def __init__(
        self,
        topConfigPath=None,
        hostName=None,
        siteLoc=None,
        siteId=None,
        cacheFlag=True,
        verbose=True,
        log=sys.stdout,  # noqa: ARG002
    ):  # noqa: ARG002 pylint: disable=unused-argument
        self.__lfh = log
        self.__debug = False
        self.__siteId = None
        self.__siteLoc = None
        self.__topConfigPath = None
        self.__cD = {}
        #
        # Complete list of sections maintained as private namespaces
        self.__privateSectionNameList = [
            "os_environment",
            "httpd_services",
            "install_environment",
            "database_services",
            "validation_services",
        ]
        #
        # additional configuration sections added to the common namespace
        self.__extraCommonSectionNameList = ["database_services", "validation_services"]

        if topConfigPath is None:
            topConfigPath = os.getenv("TOP_WWPDB_SITE_CONFIG_DIR", default=None)
        ok = self.__testConfigPath(topConfigPath)
        if ok:
            self.__topConfigPath = topConfigPath
            self.__siteLoc, self.__siteId = self.__setup(topConfigPath, hostName, siteLoc, siteId)

            if cacheFlag:
                self.__cD = self.__getConfigD(self.__topConfigPath, self.__siteLoc, self.__siteId)
            else:
                self.__cD = self.__getSiteConfigRaw(self.__topConfigPath, self.__siteLoc, self.__siteId)

    def __setup(self, topConfigPath, inpHostName, inpSiteLoc, inpSiteId):
        """ """
        siteLoc = None
        siteId = None
        if topConfigPath is None:
            self.__lfh.write("FAILING - missing configuration file path\n")
        elif inpSiteLoc is not None and inpSiteId is not None:
            #
            # load site configuration data cache -
            siteLoc = inpSiteLoc
            siteId = inpSiteId
        elif inpHostName is not None:
            # read host mapping data
            fp = self.__getCommonConfigPath(topConfigPath)
            cD = self.__readConfigTextFile(fp)
            if "HOST_SITE_DEFAULTS" in cD:
                hnU = str(inpHostName).upper()
                if hnU in cD["HOST_SITE_DEFAULTS"]:
                    tL = cD["HOST_SITE_DEFAULTS"][hnU].split(",")
                    siteLoc = tL[0]
                    siteId = tL[1]
        else:
            self.__lfh.write("FAILING configuration could not be resolved\n")
        if self.__debug:
            self.__lfh.write("_setup returns siteLoc %r siteId %r\n" % (siteLoc, siteId))
        return siteLoc, siteId

    def __getPrivateSectionNames(self):
        return self.__privateSectionNameList

    def __getExtraCommonSectionNames(self):
        return self.__extraCommonSectionNameList

    def __getConfigD(self, topConfigPath, siteLoc, siteId):
        """Load the current python cache configuration data for the input location/site
        and return a dictionary of this data.
        """
        tD = {}
        try:
            fp = self.__getSitePythonCachePath(topConfigPath, siteLoc, siteId)
            if sys.version_info[0] > 2:
                # Assumes > python 3.4 - import.machinery.SourceFileLoader
                # would be needed
                spec = importlib.util.spec_from_file_location("ConfigInfoFileCache", fp)
                oD = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(oD)
                cD = oD.ConfigInfoFileCache._configD  # noqa: SLF001 # pylint: disable=protected-access
            else:
                oD = imp.load_source("ConfigInfoFileCache", fp)
                cD = oD.ConfigInfoFileCache._configD  # noqa: SLF001 # pylint: disable=protected-access
            tD = cD[siteId]
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("failing %s\n" % str(e))
            if self.__debug:
                traceback.print_exc(file=self.__lfh)
        return tD

    def __testConfigPath(self, topConfigPath, accessType="read"):
        ok = True
        try:
            if topConfigPath is None:
                ok = False
                self.__lfh.write("WARNING - TOP_WWPDB_SITE_CONFIG_DIR is not set in the environment.\n")
            elif accessType == "write" and not os.access(topConfigPath, os.W_OK):
                ok = False
                self.__lfh.write("WARNING - %s lacks write access\n" % topConfigPath)
            elif accessType == "read" and not os.access(topConfigPath, os.R_OK):
                ok = False
                self.__lfh.write("WARNING - %s lacks read access\n" % topConfigPath)
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("__testConfigPath failing %r\n" % str(e))
            traceback.print_exc(file=self.__lfh)
            ok = False

        return ok

    @staticmethod
    def __getCommonConfigPath(topConfigPath, sectionName="common", context="common"):
        cfPath = os.path.join(topConfigPath, "common", "common.cfg")
        return cfPath, sectionName, context

    @staticmethod
    def __getSiteCommonConfigPath(topConfigPath, siteLoc, sectionName="site_common", context="common"):
        cfPath = os.path.join(topConfigPath, siteLoc.lower(), "site_common", "common.cfg")
        return cfPath, sectionName, context

    @staticmethod
    def __getSiteConfigPath(topConfigPath, siteLoc, siteId, sectionName, context="common"):
        cfPath = os.path.join(topConfigPath, siteLoc.lower(), siteId.lower(), "site.cfg")
        return cfPath, sectionName, context

    @staticmethod
    def __getSitePythonCachePath(topConfigPath, siteLoc, siteId):
        cfPath = os.path.join(topConfigPath, siteLoc.lower(), siteId.lower(), "ConfigInfoFileCache.py")
        return cfPath

    # def __getSiteJsonCachePath(self, topConfigPath, siteLoc, siteId):
    #     cfPath = os.path.join(topConfigPath, siteLoc.lower(), siteId.lower(), "ConfigInfoFileCache.json")
    #     return cfPath

    def __readConfigTextFile(self, configFilePath):
        """Read the input configuration file and return a dictionary of configuration items
        where all configuration keys are converted to upper case.  The returned dictionary is
        organized in configuration sections (e.g. retD[sectionN.upper()]={k1:v1,k2:v2,...})

        """
        retD = {}
        try:
            if sys.version_info[0] > 2:
                # Python 3.2 deprecates SafeConfigParser()
                config = ConfigParser.ConfigParser()
            else:
                config = ConfigParser.SafeConfigParser()
            # print configFilePath
            config.read(configFilePath)
            sectionL = config.sections()
            # print sectionL
            for section in sectionL:
                kvTupL = config.items(section)
                sKyU = section.upper()
                d = {}
                for k, v in kvTupL:
                    d[k.upper()] = v
                retD[sKyU] = d
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("FAILED reading %s - %s\n" % (configFilePath, str(e)))
            if self.__debug:
                traceback.print_exc(file=self.__lfh)

        return retD

    def __getConfigPathSectionList(
        self, topConfigPath, siteLoc, siteId, extraCommonSectionNameList, privateSectionNameList
    ):
        """Returns the search path of sections and configuration file paths for the input location and site.
        The site specific configuration file path is always included.   The site-common or project common
        configuration files are included only if these exist.

        Any section names in extraCommonSectionNameList are added to path list for each configuration file -

        Returns: [(configPath,sectionName,context), (configPath,sectionName),context), ...]
        """
        cfPathSectionList = []
        if self.__topConfigPath is not None and siteId is not None and siteLoc is not None:
            (p, s, c) = self.__getSiteConfigPath(
                topConfigPath=topConfigPath, siteLoc=siteLoc, siteId=siteId, sectionName=siteId.lower()
            )
            if p is not None and os.access(p, os.R_OK):
                cfPathSectionList.append((p, s, c))
                for cSec in extraCommonSectionNameList:
                    cfPathSectionList.append((p, cSec, "common"))
            (p, s, c) = self.__getSiteCommonConfigPath(
                topConfigPath=topConfigPath, siteLoc=siteLoc, sectionName="site_common"
            )
            if p is not None and os.access(p, os.R_OK):
                cfPathSectionList.append((p, s, c))
                for cSec in extraCommonSectionNameList:
                    cfPathSectionList.append((p, cSec, "common"))
            (p, s, c) = self.__getCommonConfigPath(topConfigPath=topConfigPath, sectionName="common")
            if p is not None and os.access(p, os.R_OK):
                cfPathSectionList.append((p, s, c))
                for cSec in extraCommonSectionNameList:
                    cfPathSectionList.append((p, cSec, "common"))
            #
            # Additional context specific (private) configuration sections - stored in the site specific path
            #
            for sectionName in privateSectionNameList:
                (p, s, c) = self.__getSiteConfigPath(
                    topConfigPath=topConfigPath,
                    siteLoc=siteLoc,
                    siteId=siteId,
                    sectionName=sectionName,
                    context="private",
                )
                if p is not None and os.access(p, os.R_OK):
                    cfPathSectionList.append((p, s, c))

        return cfPathSectionList

    def __getSiteConfigRaw(self, topConfigPath, siteLoc, siteId, deserialize=True):
        """Return the complete site of configuration options for the input location and site."""
        cD = {}
        try:
            privateSectionNameList = self.__getPrivateSectionNames()
            extraCommonSectionNameList = self.__getExtraCommonSectionNames()
            pathSectList = self.__getConfigPathSectionList(
                topConfigPath, siteLoc, siteId, extraCommonSectionNameList, privateSectionNameList
            )
            if self.__debug:
                self.__lfh.write(
                    "__getSiteConfigRaw location %r site %r path list %r \n" % (siteLoc, siteId, pathSectList)
                )
            cD = self.__readConfigFileList(configPathSectionList=pathSectList)
            if deserialize:
                cD = self.__deserializeConfig(cD, optionD=cD)
                for sectionName in privateSectionNameList:
                    sU = sectionName.upper()
                    if sU in cD:
                        cD[sU] = self.__deserializeConfig(cD[sU], optionD=cD[sU])
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("__getSiteConfigRaw failing for location %r site %r - %s\n" % (siteLoc, siteId, str(e)))
            traceback.print_exc(file=self.__lfh)
        return cD

    def __readConfigFileList(self, configPathSectionList=None):
        """Read the input list of configuration file paths/section names   [(configPath,sectionName,context), (configPath,sectionName,context),].
        Preceding files in this list may supply substition values for subsequent files through string interpolation
        such as %(replace_me)s.  The first instance of any option/value encountered in the path list is treated as authoritative.

        The ConfigParser class processes all configuration option values as strings.  Section names are searched in lower case.

        Returns configuration options from all sections in a dictionary with option keys in upper case.

        """
        retD = {}
        try:
            defaultD = {}
            saveD = {}
            # For each configuration file in turn -- accumulated content provides default value substition values for subsequent files -
            #
            # Template substitution performed explicitly here using any preceding content in the 'common' namespace --
            if configPathSectionList is not None:
                for configFilePath, sectionName, context in configPathSectionList:
                    config = ConfigParser.RawConfigParser(allow_no_value=True)
                    config.read(configFilePath)
                    sectionL = config.sections()
                    if sectionName in sectionL:
                        kvTupL = config.items(sectionName.lower())
                        # for k, v in kvTupL:
                        #    defaultD[k] = v
                        if self.__debug:
                            self.__lfh.write(
                                "__readConfigFileList fetching section %s length %d\n" % (sectionName, len(kvTupL))
                            )
                        if context == "common":
                            for k, v in kvTupL:
                                # Respect existing values in the order of config files -
                                if k not in saveD:
                                    try:
                                        saveD[k] = v % defaultD
                                    except BaseException as e:  # noqa: BLE001
                                        self.__lfh.write("substitution failed for %r %r %r\n" % (k, v, str(e)))
                                        continue
                                    # update substitution defaults ...
                                    defaultD[k] = saveD[k]
                        elif context == "private":
                            pD = {}
                            pDU = {}
                            for k, v in kvTupL:
                                if k not in pD:
                                    try:
                                        pD[k] = v % defaultD
                                    except BaseException as e:  # noqa: BLE001
                                        self.__lfh.write("substitution failed for %r %r %r\n" % (k, v, str(e)))
                                        continue
                                    # update substitution defaults ...
                                    defaultD[k] = pD[k]
                                    pDU[k.upper()] = pD[k]
                            saveD[sectionName.upper()] = pDU
                        for k, v in saveD.items():
                            defaultD[k] = v

            # Copy the accumulated saved items for return with upper-cased keys --
            for k, v in saveD.items():
                retD[k.upper()] = v
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("failed reading configuration file list %r %r\n" % (configPathSectionList, str(e)))
            if self.__debug:
                traceback.print_exc(file=self.__lfh)

        return retD

    def __deserializeConfig(self, configD, optionD=None):
        """Apply an adhoc set of filters on the input configuration dictionary.
        Input option values are assumed to be the string values returned by the configuration file parser.

        The following inline options are used as filter selectors:

        config_as_object - "option, option, option"
                           convert string to object. For instance, create tuple, list or dict from
                           string (e.g __repr__) represenation. This also handles "None" -> None

        config_csv_as_list - "option, option, option"
                              convert comma separated values to a list of strings

        config_csv_as_int_list - "option, option, option"
                               convert comma separated values to a list of integers

        config_as_int|float  - "option, option, option"
                              convert string to int|float

        all values are tested for the literal 'None' string which is converted to a None value.

        All input option keys (optionD) are processed with leading and trailing whitespace stripped and
        in upper case.

        Returns an updated dictionary of configuration options with values cast according filter conditions.

        """
        retD = {}
        objD = {}
        lstD = {}
        intD = {}
        fltD = {}
        iLstD = {}
        try:
            if optionD is not None:
                optD = dict((k.lower(), v) for k, v in optionD.items())  # noqa: C402
                if "config_as_object" in optD:
                    objD = dict.fromkeys(
                        [t.strip().upper() for t in optD["config_as_object"].split(",") if len(t.strip()) > 0]
                    )
                if "config_csv_as_list" in optD:
                    lstD = dict.fromkeys(
                        [t.strip().upper() for t in optD["config_csv_as_list"].split(",") if len(t.strip()) > 0]
                    )
                if "config_as_int" in optD:
                    intD = dict.fromkeys(
                        [t.strip().upper() for t in optD["config_as_int"].split(",") if len(t.strip()) > 0]
                    )
                if "config_as_float" in optD:
                    fltD = dict.fromkeys(
                        [t.strip().upper() for t in optD["config_as_float"].split(",") if len(t.strip()) > 0]
                    )
                if "config_csv_as_int_list" in optD:
                    iLstD = dict.fromkeys(
                        [t.strip().upper() for t in optD["config_csv_as_int_list"].split(",") if len(t.strip()) > 0]
                    )
            #
            # if self.__debug:
            #   print "Filter as object", objD
            #
            for k, v in configD.items():
                retD[k] = v
                if v == "None":
                    retD[k] = None

                if k in intD:
                    retD[k] = int(v)

                if k in fltD:
                    retD[k] = float(v)

                try:
                    if k in lstD:
                        retD[k] = [t.strip() for t in v.split(",")]
                    if k in iLstD:
                        retD[k] = [int(t.strip()) for t in v.split(",")]
                except Exception as e:  # noqa: BLE001
                    self.__lfh.write("failed csv list filter %r %r %r\n" % (k, v, str(e)))
                try:
                    if k in objD:
                        retD[k] = ast.literal_eval(v)
                except Exception as e:  # noqa: BLE001
                    self.__lfh.write("failed eval filter %r %r %r\n" % (k, v, str(e)))
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("failed configuration filter %r\n" % str(e))
            if self.__debug:
                traceback.print_exc(file=self.__lfh)

        return retD

    def printConfig(self):
        return self.__printConfig(self.__siteLoc, self.__siteId, self.__cD)

    def __printConfig(self, siteLoc, siteId, cD):
        """Print the configuration options for the input location and site."""
        try:
            self.__lfh.write("read %d options for location %r site %r\n" % (len(cD), siteLoc, siteId))
            for k in sorted(cD.keys()):
                v = cD[k]
                if type(v) == dict:  # noqa: E721
                    self.__lfh.write(" +++ %-45s  %r\n" % (k, "private context"))
                    for k1 in sorted(v.keys()):
                        self.__lfh.write(" ---  --- +++ %-45s  %r\n" % (k1, v[k1]))
                else:
                    self.__lfh.write(" +++ %-45s  %r\n" % (k, v))
        except Exception as e:  # noqa: BLE001
            self.__lfh.write("failing for location %r site %r %r\n" % (siteLoc, siteId, str(e)))
            if self.__debug:
                traceback.print_exc(file=self.__lfh)

    def shellConfig(self, shellType="bash"):
        return self.__exportConfig(
            self.__siteLoc, self.__siteId, self.__cD, expKey="OS_ENVIRONMENT", shellType=shellType
        )

    def httpdConfig(self, shellType="bash"):
        return self.__exportConfig(
            self.__siteLoc, self.__siteId, self.__cD, expKey="HTTPD_SERVICES", shellType=shellType
        )

    def installConfig(self, shellType="bash"):
        return self.__exportConfig(
            self.__siteLoc, self.__siteId, self.__cD, expKey="INSTALL_ENVIRONMENT", shellType=shellType
        )

    def validationConfig(self, shellType="bash"):
        return self.__exportConfig(
            self.__siteLoc, self.__siteId, self.__cD, expKey="VALIDATION_SERVICES", shellType=shellType
        )

    def databaseConfig(self, shellType="bash"):
        return self.__exportConfig(
            self.__siteLoc, self.__siteId, self.__cD, expKey="DATABASE_SERVICES", shellType=shellType
        )

    def __exportConfig(self, siteLoc, siteId, cD, expKey="OS_ENVIRONMENT", shellType="bash"):
        """Print the configuration options for the input location and site."""
        try:
            if expKey in cD:
                dd = cD[expKey]
                for k in sorted(dd.keys()):
                    v = dd[k]
                    if shellType in ["bash", "sh"]:
                        self.__lfh.write('export %s="%s"\n' % (k, v))
                    elif shellType in ["csh", "tcsh"]:
                        self.__lfh.write('setenv %s "%s"\n' % (k, v))
        except Exception as e:  # noqa: BLE001
            if self.__debug:
                self.__lfh.write("__exportConfig failing for location %r site %r - %r\n" % (siteLoc, siteId, str(e)))
                traceback.print_exc(file=self.__lfh)


def main():  # pragma: no cover
    usage = """
    %prog [options]

    Examples:

     Export shell configuration options  (requires both --locid & --siteid):

       python %prog --shell --configpath=/wwpdb_da/site-config --siteid=WWPDB_DEPLOY_TEST_RU --locid=rcsb-east

     Export shell configuration options for the default host site assignment:

       python %prog --shell --configpath=/wwpdb_da/site-config --hostname=myhost.wwpdb.org --shellType='bash'

     Print the all configuration options  (requires both --locid & --siteid or --hostname):

       python %prog --print --configpath=/wwpdb_da/site-config --siteid=WWPDB_DEPLOY_TEST_RU --locid=rcsb-east

    """
    parser = OptionParser(usage)
    parser.add_option(
        "--print",
        dest="printConfig",
        action="store_true",
        default=False,
        help="Print the configuration options for site (--siteid) at location (--locid)",
    )
    parser.add_option("--hostname", dest="hostName", default=None, help="Fully qualified host name")
    parser.add_option("--siteid", dest="siteId", default=None, help="wwPDB site ID (e.g. WWPDB_DEPLOY_TEST_RU)")
    parser.add_option(
        "--locid", dest="siteLoc", default=None, help="wwPDB location ID (e.g. pdbe, pdbj, rcsb-east, ... )"
    )
    parser.add_option(
        "--configpath", dest="topConfigPath", default=None, help="Configuration path (e.g. /wwpdb_da/site-config)"
    )

    parser.add_option(
        "--shell",
        dest="shellConfig",
        action="store_true",
        default=False,
        help="Export shell environment for (--siteid) at location (--locid) or host",
    )
    parser.add_option("--shelltype", dest="shellType", default="bash", help="Export shell type ('bash', 'csh')")
    parser.add_option(
        "--httpd",
        dest="httpdConfig",
        action="store_true",
        default=False,
        help="Export httpd environment for (--siteid) at location (--locid) or host",
    )
    parser.add_option(
        "--install",
        dest="installConfig",
        action="store_true",
        default=False,
        help="Export installation environment for (--siteid) at location (--locid) or host",
    )
    parser.add_option(
        "--validation",
        dest="validationConfig",
        action="store_true",
        default=False,
        help="Export validation service environment for (--siteid) at location (--locid) or host",
    )
    parser.add_option(
        "--database",
        dest="databaseConfig",
        action="store_true",
        default=False,
        help="Export database services environment for (--siteid) at location (--locid) or host",
    )
    parser.add_option("-v", "--verbose", default=True, action="store_true", dest="verbose")
    parser.add_option("--nocache", default=False, action="store_true", dest="nocacheFlag")

    (options, _args) = parser.parse_args()  # pylint: disable=unused-variable

    if options.topConfigPath is None:
        print("Configuration path must be specified (--configpath)")  # noqa: T201
        parser.print_help()
        sys.exit(-1)

    if options.hostName is None and (options.siteLoc is None or options.siteId is None):
        print("Either hostname or the combination of siteLoc and siteId must be specified")  # noqa: T201
        parser.print_help()
        sys.exit(-1)

        # parser.error(usage)

    cI = ConfigInfoShellExec(
        topConfigPath=options.topConfigPath,
        hostName=options.hostName,
        siteLoc=options.siteLoc,
        siteId=options.siteId,
        cacheFlag=not options.nocacheFlag,
        verbose=options.verbose,
        log=sys.stdout,
    )

    if options.printConfig:
        cI.printConfig()

    if options.shellConfig:
        cI.shellConfig(shellType=options.shellType)

    if options.httpdConfig:
        cI.httpdConfig(shellType=options.shellType)

    if options.installConfig:
        cI.installConfig(shellType=options.shellType)

    if options.validationConfig:
        cI.validationConfig(shellType=options.shellType)

    if options.databaseConfig:
        cI.databaseConfig(shellType=options.shellType)


if __name__ == "__main__":
    main()
